import numpy as np
import pandas
import re
import sqlite3

#--------------------------------------
# modified from thinkstats2 code
#--------------------------------------
def readStataDct(dct_file, **options):
    """Reads a Stata dictionary file.

    dct_file: string filename
    options: dict of options passed to open()

    returns: dataframe object
    """
    type_map = dict(byte='int', int='int', long='int', float='real',\
    double='real')

    var_info = []
    for line in open(dct_file, **options):
        match = re.search( r'_column\(([^)]*)\)', line)
        if match:
            start = int(match.group(1))
            t = line.split()
            vtype, name, fstring = t[1:4]
            name = name.lower()
            if vtype.startswith('str'):
                vtype = 'char'
            else:
                vtype = type_map[vtype]
            long_desc = ' '.join(t[4:]).strip('"')
            var_info.append((start, vtype, name, fstring, long_desc))
            
    columns = ['start', 'type', 'name', 'fstring', 'desc']
    variables = pandas.DataFrame(var_info, columns=columns)

    # fill in the end column by shifting the start column
    variables['end'] = variables.start.shift(-1)
    variables.loc[len(variables)-1, 'end'] = 0

    return variables

#--------------------------------------
#--------------------------------------
def getDDL(dct,tname):
    ddlstr='CREATE TABLE ' + tname + '('

    for (s,t,n,f,d,e) in dct.values:
        lenstr=''
        if t=='char':
            lenstr = '(' + f[1:-1] + ')'
        # print(n + ' ' + t + lenstr + ',')
        ddlstr += n + ' ' + t + lenstr
        if e>s:
            ddlstr += ',\n'

    ddlstr += '\n);'

    return (ddlstr)

#--------------------------------------
# adapted from 
# https://stackoverflow.com/questions/4914008/how-to-efficiently-parse-fixed-width-files
#--------------------------------------
def slices(s, colspec):
    position = 0
    fields = []
    for (width,ctype) in colspec:
        colstr = s[position:position + width]
        if colstr=='' or colstr.isspace():
            colval=None
        elif ctype=='int':
            colval = int(s[position:position + width])
        elif ctype=='real':
            colval = float(s[position:position + width])
        else:
            colval = colstr
        fields.append(colval)
        position += width
    return fields

#--------------------------------------
# load the data into a sqlite database
#--------------------------------------
def loadData(dbname, tname, datfile, dct):

    conn = sqlite3.connect(dbname)
    c = conn.cursor()

    # check: if table exists, drop table
    c.execute('SELECT sql FROM sqlite_master WHERE tbl_name = ?', (tname,))
    if c.fetchone() != None:
        c.execute('DROP TABLE '+ tname + ';')

    # create table
    c.execute(getDDL(dct,tname))

    # construct column specification : list of (widths, type)
    colspec = [(int(f[1:-1]),t) for (s,t,n,f,d,e) in dct.values]

    # construct the INSERT statement
    inssql = 'INSERT INTO '+tname+' VALUES(?'
    for i in range(dct.values.shape[0]-1):
        inssql += ',?'
    inssql += ');'

    # open data file
    nrows = 0
    with open(datfile) as f:
        # read line by line
        for line in f:
            # parse each line using the col widths
            row = slices(line, colspec)
            #print(row)
            # insert row into database
            c.execute(inssql,row)
            nrows+=1

    #print(inssql)
    conn.commit()
    conn.close()
    return nrows

# read the metadata/schema
dct = readStataDct('2013_2015_FemPregSetup.dct')

#print(dct.values)
dbname='nsfg.db'
datfile='2013_2015_FemPregData.dat'
tname = 'fempreg'

n = loadData(dbname, tname, datfile, dct)

print('Loaded ' + str(n) + ' rows from data file ' + datfile + ' into ' + dbname + '.' + tname)
